@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2024 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2025 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -1027,7 +1027,7 @@ int PWScore::ImportPlaintextFile(const StringX &ImportedPrefix,
     Format(cs_error, IDSC_IMPORTHDR, numCols);
     rpt.WriteLine(cs_error);
     LoadAString(cs_error, bImportPSWDsOnly ? IDSC_IMPORTKNOWNHDRS2 : IDSC_IMPORTKNOWNHDRS);
-    rpt.WriteLine(cs_error, bImportPSWDsOnly);
+    rpt.WriteLine(cs_error);
     for (auto fieldName = foundFields.begin(); fieldName != foundFields.end(); ++fieldName)
     {
       if (*fieldName == CItemData::EngFieldName(CItem::UNKNOWNFIELDS))
@@ -1038,12 +1038,14 @@ int PWScore::ImportPlaintextFile(const StringX &ImportedPrefix,
 
     if (!unknownColumns.empty())
     {
+      rpt.WriteLine();
       LoadAString(cs_error, IDSC_UNKNOWNHDRS);
-      rpt.WriteLine(cs_error, false);
+      rpt.WriteLine(cs_error);
     }
     for (const auto &unknownColumn : unknownColumns)
     {
-      rpt.WriteLine(unknownColumn, false);
+      Format(cs_error, L"\t%ls ", unknownColumn.c_str());
+      rpt.WriteLine(cs_error);
     }
     rpt.WriteLine();
     rpt.WriteLine();
@@ -1552,6 +1554,13 @@ int PWScore::ImportPlaintextFile(const StringX &ImportedPrefix,
              SUCCESS : OK_WITH_ERRORS;
 }
 
+    // Remove trailing \r, if any [GH1560]
+static inline void trimCR(std::string &str) {
+  if (!str.empty() && str.back() == '\r') {
+    str.pop_back();
+  }
+}
+
 int PWScore::ImportKeePassV1TXTFile(const StringX &filename,
                                     int &numImported, int &numSkipped, int &numRenamed,
                                     UINT &uiReasonCode, CReport &rpt, Command *&pcommand)
@@ -1560,7 +1569,7 @@ int PWScore::ImportKeePassV1TXTFile(const StringX &filename,
   /*
   The format of the source file is from doing an export to TXT file in Keepass.
 
-  The checkbox "Encode/replace newline characters by '\n'" MUST be selected bu the
+  The checkbox "Encode/replace newline characters by '\n'" MUST be selected by the
   user during the export or this import will fail and may give unexpected results.
 
   The line that starts with '[' and ends with ']' is equivalent to the Title field.
@@ -1666,6 +1675,8 @@ int PWScore::ImportKeePassV1TXTFile(const StringX &filename,
       bFirst = false;
     }
 
+    trimCR(linebuf);
+
     // this line should always be a title contained in []'s
     if (*(linebuf.begin()) != '[' || *(linebuf.end() - 1) != ']') {
       LoadAString(cs_error, IDSC_IMPORTMISSINGTITLE);
@@ -1682,10 +1693,13 @@ int PWScore::ImportKeePassV1TXTFile(const StringX &filename,
     for (;;) {
       streamoff currentpos = iss.tellg();
       getline(iss, linebuf, '\n');
+      
 
       if (iss.eof())
         break;
 
+      trimCR(linebuf);
+
       // Check if blank line
       if (linebuf.empty())
         continue;
@@ -2075,6 +2089,7 @@ int PWScore::ImportKeePassV1CSVFile(const StringX &filename,
     }
   }
 
+  trimCR(s_header);
   // Parse the header line
   std::vector<StringX> hdr_tokens;
   ProcessKeePassCSVLine(s_header, hdr_tokens);
@@ -2158,6 +2173,8 @@ int PWScore::ImportKeePassV1CSVFile(const StringX &filename,
     if (iss.eof())
       break;
 
+    trimCR(linebuf);
+
     // skip blank lines
     if (linebuf.empty()) {
       Format(strError, IDSC_IMPORTEMPTYLINESKIPPED, numlines);
