@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2024 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2025 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -848,7 +848,7 @@ stringT PWScore::Merge(PWScore *pothercore,
   if (numAliasesAdded > 0 && pRpt != nullptr) {
     std::sort(vs_AliasesAdded.begin(), vs_AliasesAdded.end(), MergeSyncGTUCompare);
     stringT str_singular_plural_type, str_singular_plural_verb;
-    LoadAString(str_singular_plural_type, numAliasesAdded == 1 ? IDSC_ENTRY : IDSC_ENTRIES);
+    LoadAString(str_singular_plural_type, numAliasesAdded == 1 ? IDSC_ALIAS : IDSC_ALIASES);
     LoadAString(str_singular_plural_verb, numAliasesAdded == 1 ? IDSC_WAS : IDSC_WERE);
     Format(str_results, IDSC_MERGEADDED, str_singular_plural_type.c_str(),
                     str_singular_plural_verb.c_str());
@@ -862,7 +862,7 @@ stringT PWScore::Merge(PWScore *pothercore,
   if (numShortcutsAdded > 0 && pRpt != nullptr) {
     std::sort(vs_ShortcutsAdded.begin(), vs_ShortcutsAdded.end(), MergeSyncGTUCompare);
     stringT str_singular_plural_type, str_singular_plural_verb;
-    LoadAString(str_singular_plural_type, numShortcutsAdded == 1 ? IDSC_ENTRY : IDSC_ENTRIES);
+    LoadAString(str_singular_plural_type, numShortcutsAdded == 1 ? IDSC_SHORTCUT : IDSC_SHORTCUTS);
     LoadAString(str_singular_plural_verb, numShortcutsAdded == 1 ? IDSC_WAS : IDSC_WERE);
     Format(str_results, IDSC_MERGEADDED, str_singular_plural_type.c_str(),
                     str_singular_plural_verb.c_str());
@@ -998,6 +998,46 @@ int PWScore::MergeDependents(PWScore *pothercore, MultiCommands *pmulticmds,
   return numadded;
 }
 
+bool PWScore::SyncItem(const CItemData& srcItem, CItemData& dstItem, const CItemData::FieldBits& bsFields,
+                       MultiCommands& mcmd, const PWScore *potherCore)
+{
+  bool bUpdated(false);
+  // Do not try and change GROUPTITLE = 0x00 (use GROUP & TITLE separately) or UUID = 0x01
+  for (size_t i = 2; i < bsFields.size(); i++) {
+
+    if (bsFields.test(i)) {
+      const CItem::FieldType ft = static_cast<CItem::FieldType>(i);
+      const StringX sxValue = srcItem.GetFieldValue(ft);
+
+      // Special processing for password policies (default & named)
+      if (ft == CItemData::POLICYNAME) {
+        // Don't really need the map and vector as only sync'ing 1 entry
+        std::map<StringX, StringX> mapRenamedPolicies;
+        std::vector<StringX> vs_PoliciesAdded;
+
+        const StringX sxSync_DateTime = PWSUtil::GetTimeStamp(true).c_str();
+        StringX sxPolicyName = srcItem.GetPolicyName();
+
+        Command* pPolicyCmd = ProcessPolicyName(potherCore, dstItem,
+          mapRenamedPolicies, vs_PoliciesAdded, sxPolicyName, bUpdated,
+          sxSync_DateTime, IDSC_SYNCPOLICY);
+        if (pPolicyCmd != nullptr)
+          mcmd.Add(pPolicyCmd);
+      } else {
+        if (sxValue != dstItem.GetFieldValue(ft)) {
+          bUpdated = true;
+          if (!CItem::IsTimeField(ft))
+            dstItem.SetFieldValue(ft, sxValue);
+          else
+            dstItem.CopyTime(ft, srcItem); // avoid hassle of parsing locale-time representations
+        }
+      }
+    }
+  }
+  return bUpdated;
+}
+
+
 void PWScore::Synchronize(PWScore *pothercore,
                           const CItemData::FieldBits &bsFields, const bool &subgroup_bset,
                           const stringT &subgroup_name,
@@ -1093,10 +1133,11 @@ void PWScore::Synchronize(PWScore *pothercore,
       // Do not try and change GROUPTITLE = 0x00 (use GROUP & TITLE separately) or UUID = 0x01
       for (size_t i = 2; i < bsSyncFields.size(); i++) {
         if (bsSyncFields.test(i)) {
-          StringX sxValue = otherItem.GetFieldValue(static_cast<CItemData::FieldType>(i));
+          const CItem::FieldType ft = static_cast<CItem::FieldType>(i);
+          StringX sxValue = otherItem.GetFieldValue(ft);
 
           // Special processing for password policies (default & named)
-          if (static_cast<CItemData::FieldType>(i) == CItemData::POLICYNAME) {
+          if (ft == CItemData::POLICYNAME) {
             Command *pPolicyCmd = ProcessPolicyName(pothercore, updItem,
                                    mapRenamedPolicies, vs_PoliciesAdded,
                                    sxValue, bUpdated,
@@ -1104,9 +1145,12 @@ void PWScore::Synchronize(PWScore *pothercore,
             if (pPolicyCmd != nullptr)
               pmulticmds->Add(pPolicyCmd);
           } else {
-            if (sxValue != updItem.GetFieldValue(static_cast<CItemData::FieldType>(i))) {
+            if (sxValue != updItem.GetFieldValue(ft)) {
               bUpdated = true;
-              updItem.SetFieldValue(static_cast<CItemData::FieldType>(i), sxValue);
+              if (!CItem::IsTimeField(ft))
+                updItem.SetFieldValue(ft, sxValue);
+              else
+                updItem.CopyTime(ft, otherItem); // avoid hassle of parsing locale-time representations
             }
           }
         }
@@ -1175,7 +1219,7 @@ void PWScore::Synchronize(PWScore *pothercore,
   pRpt->WriteLine(str_results.c_str());
 }
 
-Command *PWScore::ProcessPolicyName(PWScore *pothercore, CItemData &updtEntry,
+Command *PWScore::ProcessPolicyName(const PWScore *pothercore, CItemData &updtEntry,
                                     std::map<StringX, StringX> &mapRenamedPolicies,
                                     std::vector<StringX> &vs_PoliciesAdded,
                                     StringX &sxOtherPolicyName, bool &bUpdated,
