@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2024 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2025 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -208,6 +208,7 @@ public:
   void SetUser(const StringX &user) {CItem::SetField(USER, user);} // V20
   void SetPassword(const StringX &password) {CItem::SetField(PASSWORD, password);}
   void UpdatePassword(const StringX &password); // use when password changed!
+  void ClearTwoFactorKey() {ClearField(CItemData::TWOFACTORKEY);}
   void SetTwoFactorKey(const StringX& value) { CItem::SetField(TWOFACTORKEY, value); }
   bool SetTotpConfig(const StringX& value) { return SetFieldAsByte(TOTPCONFIG, value.c_str()); }
   void SetTotpStartTime(time_t t) { CItem::SetTime(TOTPSTARTTIME, t); }
@@ -352,6 +353,57 @@ public:
   bool IsURLEmail(const CItemData *pbci) const
   { return GetEffectiveFieldValue(URL, pbci).find(_T("mailto:")) != StringX::npos; }
 
+  // V3 attachment
+  bool IsAttTitleSet() const            { return IsFieldSet(DATA_ATT_TITLE);     }
+  bool IsAttMediaTypeSet() const        { return IsFieldSet(DATA_ATT_MEDIATYPE); }
+  bool IsAttFileNameSet() const         { return IsFieldSet(DATA_ATT_FILENAME);  }
+  bool IsAttModificationTimeSet() const { return IsFieldSet(DATA_ATT_MTIME);     }
+  bool IsAttContentSet() const          { return IsFieldSet(DATA_ATT_CONTENT);   }
+
+  StringX GetAttTitle() const           { return GetField(DATA_ATT_TITLE);     }
+  StringX GetAttMediaType() const       { return GetField(DATA_ATT_MEDIATYPE); }
+  StringX GetAttFileName() const        { return GetField(DATA_ATT_FILENAME);  }
+  time_t GetAttModificationTime(time_t &t) const { CItem::GetTime(DATA_ATT_MTIME, t); return t; }
+  size_t GetAttContentLength() const;
+  std::vector<unsigned char> GetAttContent() const;
+
+  void SetAttTitle(const StringX &title)         { CItem::SetField(DATA_ATT_TITLE, title); }
+  void SetAttMediaType(const StringX &mediaType) { CItem::SetField(DATA_ATT_MEDIATYPE, mediaType); }
+  void SetAttFileName(const StringX &fileName)   { CItem::SetField(DATA_ATT_FILENAME, fileName); }
+  void SetAttModificationTime(time_t t)          { CItem::SetTime(DATA_ATT_MTIME, t); }
+  void SetAttContent(const unsigned char *content, size_t clen) { CItem::SetField(DATA_ATT_CONTENT, content, clen); }
+
+  // Following works for both V3 and V4 attachments
+  bool HasAttachment() const            { return IsAttMediaTypeSet() || HasAttRef(); }
+  // Following is V3-specific, since clearing a V4 attachment also requires removing a CItemAtt object from core.
+  void ClearV3Attachment();
+
+  // Passkey
+  bool IsPasskeyCredentialIDSet() const    { return IsFieldSet(PASSKEY_CRED_ID);     }
+  bool IsPasskeyRelyingPartyIDSet() const  { return IsFieldSet(PASSKEY_RP_ID);       }
+  bool IsPasskeyUserHandleSet() const      { return IsFieldSet(PASSKEY_USER_HANDLE); }
+  bool IsPasskeyAlgorithmIDSet() const     { return IsFieldSet(PASSKEY_ALGO_ID);     }
+  bool IsPasskeyPrivateKeySet() const      { return IsFieldSet(PASSKEY_PRIVATE_KEY); }
+  bool IsPasskeySignCountSet() const       { return IsFieldSet(PASSKEY_SIGN_COUNT);  }
+
+  StringX GetPasskeyRelyingPartyID() const { return GetField(PASSKEY_RP_ID);         }
+  int32 GetPasskeyAlgorithmID() const;
+  uint32 GetPasskeySignCount() const;
+  VectorX<unsigned char> GetPasskeyCredentialID() const;
+  VectorX<unsigned char> GetPasskeyUserHandle() const;
+  VectorX<unsigned char> GetPasskeyPrivateKey() const;
+
+  void SetPasskeyRelyingPartyID(const StringX &rp_id) { CItem::SetField(PASSKEY_RP_ID, rp_id); }
+  void SetPasskeyAlgorithmID(const int32 algo_id);
+  void SetPasskeySignCount(const uint32 sign_count);
+  void SetPasskeyCredentialID(const VectorX<unsigned char> &v) { CItem::SetField(PASSKEY_CRED_ID, v.data(), v.size());     };
+  void SetPasskeyUserHandle(const VectorX<unsigned char> &v)   { CItem::SetField(PASSKEY_USER_HANDLE, v.data(), v.size()); };
+  void SetPasskeyPrivateKey(const VectorX<unsigned char> &v)   { CItem::SetField(PASSKEY_PRIVATE_KEY, v.data(), v.size()); };
+
+  bool HasPasskey() const                  { return IsPasskeyPrivateKeySet();        }
+  bool HasIncompletePasskey() const;
+  void ClearPasskey();
+
 private:
   EntryType m_entrytype;
   EntryStatus m_entrystatus;
@@ -388,6 +440,9 @@ inline bool CItemData::IsTextField(unsigned char t)
     t == KBSHORTCUT || t == ATTREF || t == BASEUUID || t == ALIASUUID ||
     t == SHORTCUTUUID ||
     t == TOTPCONFIG || t == TOTPLENGTH || t == TOTPSTARTTIME || t == TOTPTIMESTEP ||
+    t == DATA_ATT_MTIME || t == DATA_ATT_CONTENT ||
+    t == PASSKEY_CRED_ID || t == PASSKEY_USER_HANDLE || t == PASSKEY_ALGO_ID ||
+    t == PASSKEY_PRIVATE_KEY || t == PASSKEY_SIGN_COUNT ||
     t >= LAST_DATA);
 }
 #endif /* __ITEMDATA_H */
