@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2024 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2025 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -25,7 +25,7 @@
 
 #include "coredefs.h"
 
-// Parameter list for ParseBaseEntryPWD
+// Parameter list for ParseAliasPassword
 struct BaseEntryParms {
   // All fields except "InputType" are 'output'.
   StringX csPwdGroup;
@@ -37,7 +37,8 @@ struct BaseEntryParms {
   int ibasedata;
   bool bMultipleEntriesFound;
 
-  BaseEntryParms() : base_uuid(pws_os::CUUID::NullUUID()) {}
+  BaseEntryParms() : base_uuid(pws_os::CUUID::NullUUID()), InputType(CItemData::ET_INVALID), TargetType(CItemData::ET_INVALID)
+    {}
 };
 
 // Formatted Database properties
@@ -126,7 +127,7 @@ public:
   void ReInit(bool bNewfile = false);
 
   // Following used to read/write databases and Get/Set file name
-  bool IsDbOpen() const { return !m_currfile.empty(); }
+  bool IsDbFileSet() const { return !m_currfile.empty(); }
   StringX GetCurFile() const {return m_currfile;}
   void SetCurFile(const StringX &file) {m_currfile = file;}
 
@@ -191,6 +192,11 @@ public:
                    const int &subgroup_object, const int &subgroup_function,
                    int &numUpdated, CReport *pRpt, bool *pbCancel = nullptr);
 
+  // helper function to Sync a single item, returns true if dstItem was updated.
+  // Adds ProcessPolicyName command to multiCommand if a password policy was updated.
+  bool SyncItem(const CItemData& srcItem, CItemData& dstItem,
+                const CItemData::FieldBits& bsFields, MultiCommands &mcmd, const PWScore *potherCore);
+
   // Used for Empty Groups during Merge
   bool MatchGroupName(const StringX &stValue, const StringX &subgroup_name,
                       const int &iFunction) const;
@@ -272,7 +278,7 @@ public:
   // Get all password policy names
   void GetPolicyNames(std::vector<stringT> &vNames) const;
   bool GetPolicyFromName(const StringX &sxPolicyName, PWPolicy &st_pp) const;
-  Command *ProcessPolicyName(PWScore *pothercore, CItemData &updtEntry,
+  Command *ProcessPolicyName(const PWScore *pothercore, CItemData &updtEntry,
                              std::map<StringX, StringX> &mapRenamedPolicies,
                              std::vector<StringX> &vs_PoliciesAdded,
                              StringX &sxOtherPolicyName, bool &bUpdated,
@@ -339,7 +345,9 @@ public:
                               UUIDVector &dependentslist, 
                               const CItemData::EntryType type);
   // Takes apart a 'special' password into its components:
-  bool ParseBaseEntryPWD(const StringX &passwd, BaseEntryParms &pl);
+  bool ParseAliasPassword(const StringX &passwd, BaseEntryParms &pl);
+  // Check an Alias
+  bool CheckAliasValidity(const BaseEntryParms& pl, const StringX& selfGTU, StringX& errmess, bool &yesNoError);
 
   const CItemData *GetBaseEntry(const CItemData *pAliasOrSC) const;
   CItemData *GetBaseEntry(const CItemData *pAliasOrSC);
@@ -449,11 +457,13 @@ public:
   void PutAtt(const CItemAtt &att) {m_attlist[att.GetUUID()] = att;}
   void RemoveAtt(const pws_os::CUUID &attuuid);
   bool HasAtt(const pws_os::CUUID &attuuid) const {return m_attlist.find(attuuid) != m_attlist.end();}
-  AttList::size_type GetNumAtts() const {return m_attlist.size();}
+  AttList::size_type GetNumAtts() const;
   std::set<StringX> GetAllMediaTypes() const;
   
 protected:
   bool m_isAuxCore; // set in c'tor, if true, never update prefs from DB.  
+  // Validate() returns true if data modified, false if all OK
+  bool Validate(const size_t iMAXCHARS, CReport *pRpt, st_ValidateResults &st_vr); // protected for unit testing
 
 private:
 
@@ -536,8 +546,7 @@ private:
                         std::vector<st_GroupTitleUser> &vGTU_INVALID_UUID,
                         std::vector<st_GroupTitleUser> &vGTU_DUPLICATE_UUID,
                         st_ValidateResults &st_vr);
-  // Validate() returns true if data modified, false if all OK
-  bool Validate(const size_t iMAXCHARS, CReport *pRpt, st_ValidateResults &st_vr);
+  
 
   void ParseDependants(); // populate data structures as needed - called in ReadFile()
   void ResetAllAliasPasswords(const pws_os::CUUID &base_uuid);
@@ -572,7 +581,6 @@ private:
   bool m_bIsReadOnly;
   bool m_bUniqueGTUValidated;
   bool m_bNotifyDB;
-  bool m_bIsOpen;
 
     PWSfileHeader m_hdr;
   StringX m_InitialDBName, m_InitialDBDesc;
