@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2024 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2025 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -11,6 +11,7 @@
 
 // Create an action report file
 
+#include "coredefs.h"
 #include "../os/typedefs.h"
 #include "StringXStream.h"
 #include <map>
@@ -27,17 +28,19 @@ public:
   {WriteLine(cs_line.c_str(), bCRLF);}
   void WriteLine(LPCTSTR tc_line, bool bCRLF = true);
   void WriteLine();
-  bool SaveToDisk();
+  bool SaveToDisk(const stringT &out_dir = L"");
   bool ReadFromDisk();
   bool PurgeFromDisk();
   bool ReportExistsOnDisk() const;
   StringX GetString() {return m_osxs.rdbuf()->str();}
   bool StringEmpty() const {return !m_osxs.rdbuf() || m_osxs.rdbuf()->str().empty();}
   const stringT GetFileName() const {return m_cs_filename;}
+  const stringT GetDatabasePath() const;
+  void AppendPasskeyValidationResults(const std::vector<st_GroupTitleUser> &incomplete);
 
-  static const std::map<int, LPCTSTR> ReportNames;
-
+  static stringT BuildPathToReport(LPCTSTR drive, LPCTSTR dir, int report_id);
 private:
+  static const std::map<int, LPCTSTR> ReportNames;
   oStringXStream m_osxs;
   stringT m_cs_filename;
   int m_iAction;
