@@ -102,7 +102,8 @@ BOOL CAddEdit_Attachment::OnInitDialog()
   GetDlgItem(IDC_ATT_FILE)->SetFont(pFont);
 
   // Check initial state
-  if (!M_pci()->HasAttRef()) {
+  if (!M_pci()->HasAttachment() && !M_attachment().HasContent() && 
+      M_attachment().GetTitle().empty() && M_attachment().GetMediaType().empty()) {
     m_attType = NO_ATTACHMENT;
   } else {
     // m_attachment() set in CAddEdit_PropertySheet::SetupInitialValues()
@@ -116,8 +117,9 @@ BOOL CAddEdit_Attachment::OnInitDialog()
       m_csMediaType.LoadString(IDS_UNKNOWN);
     }
 
+    // Get attachment's size - Note that GetContentLength() returns original size, as opposed to GetContentSize(), which returns BlockSize()
     wchar_t szFileSize[256];
-    StrFormatByteSize(M_attachment().GetContentSize(), szFileSize, 256);
+    StrFormatByteSize(M_attachment().GetContentLength(), szFileSize, 256);
     m_csSize = szFileSize;
 
     m_csFileCTime = M_attachment().GetFileCTime().c_str();
