@@ -52,6 +52,9 @@
 #define IDS_PASSWORDHELP2               5120
 #define IDS_TRANSPARENCYHELP            5121
 #define IDS_EXCLUDE_FROM_SCR_CAP_HELP   5122
+#define IDS_PWSTRNGTH_WEAK              5123
+#define IDS_PWSTRNGTH_MEDIUM            5124
+#define IDS_PWSTRNGTH_STRONG            5125
 
 // Program strings
 #define IDS_FILEERROR1                  5210
@@ -215,7 +218,7 @@
 #define IDS_ENTRIESCHANGEDSTOP          5381
 #define IDS_ENTRIESCHANGEDSAVE          5382
 #define IDS_ENTRIESRESETMAX             5383
-#define IDS_SELECTBROWSER               5384
+#define IDS_SELECTPROGRAM               5384
 #define IDS_NOIBACKUP                   5385
 #define IDS_NONE                        5386
 #define IDS_INCREMENTNUM                5387
@@ -274,19 +277,7 @@
 #define IDS_RECORDUNKNOWNFIELDS         5450
 #define IDS_IMPORTFILE                  5451
 #define IDS_EXPORTFILE                  5453
-#define IDS_ALIASNOTFOUNDA              5455
-#define IDS_ALIASNOTFOUND0A             5456
-#define IDS_ALIASNOTFOUND0B             5457
-#define IDS_ALIASNOTFOUND1A             5458
-#define IDS_ALIASNOTFOUND1B             5459
-#define IDS_ALIASNOTFOUND2A             5460
-#define IDS_ALIASNOTFOUND2B             5461
-#define IDS_ALIASNOTFOUND2C             5462
-#define IDS_ALIASNOTFOUNDZ              5463
 #define IDS_DDBASEISALIAS               5464
-#define IDS_BASEISALIAS                 5465
-#define IDS_ALIASCANTREFERTOITSELF      5466
-#define IDS_BASEHASNOTITLE              5467
 #define IDS_CLOSE_B4_CHECK              5468
 #define IDS_CONFIRM_CLOSE               5469
 #define IDS_LATEST_VERSION              5470
@@ -309,7 +300,6 @@
 #define IDS_VISIT_WEBSITE               5489
 #define IDS_ICON                        5496
 #define IDS_ABASE_MISSING               5497
-#define IDS_ABASEINVALID                5499
 #define IDS_SBASEINVALID                5500
 #define IDS_MUSTHAVETARGET              5501
 #define IDS_VIEWSHORTCUTS               5502
