@@ -2,7 +2,7 @@
  * Initial version created as 'CryptKeyEntryDlg.h'
  * by rafaelx on 2019-03-02.
  *
- * Copyright (c) 2019-2024 Rony Shapiro <ronys@pwsafe.org>.
+ * Copyright (c) 2019-2025 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -40,6 +40,11 @@ public:
 
 protected:
 
+  void CreateControls();
+  bool IsEncryptionMode() const {
+    return m_Mode == Mode::ENCRYPT;
+  }
+
   //(*Handlers(CryptKeyEntryDialog)
   void OnOk(wxCommandEvent& event);
   void OnCancel(wxCommandEvent& event);
@@ -50,10 +55,13 @@ protected:
   //*)
 
   //(*Declarations(CryptKeyEntryDialog)
-  wxTextCtrl* TextCtrlKey1;
-  wxTextCtrl* TextCtrlKey2;
+  wxTextCtrl* m_TextCtrlKey1;
+  wxTextCtrl* m_TextCtrlKey2;
   //*)
 
+  bool isCryptKeyHidden = true;
+  bool isCryptVerificationKeyHidden = true;
+
   Mode m_Mode;
   StringX m_CryptKey;
 
