@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2024 Rony Shapiro <ronys@pwsafe.org>.
+ * Copyright (c) 2003-2025 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -475,6 +475,40 @@ void PasswordSafeFrame::DoCopyUsername(CItemData &item)
   UpdateAccessTime(item);
 }
 
+/*!
+ * wxEVT_COMMAND_MENU_SELECTED event handler for ID_COPYAUTHCODE
+ */
+
+void PasswordSafeFrame::OnCopyAuthCodeClick(wxCommandEvent& evt)
+{
+  CItemData rueItem;
+  CItemData* item = GetSelectedEntry(evt, rueItem);
+  if (item != nullptr) {
+    m_TotpLastSelectedItem = item;
+    DoCopyAuthCode(item);
+    StartTotpCopyAuthCode();
+  }
+  else {
+    StopTotpCopyAuthCode();
+  }
+}
+
+void PasswordSafeFrame::DoCopyAuthCode(const CItemData *item)
+{
+  auto totp = GetTotpData(item);
+  Clipboard::GetInstance()->SetData(totp.first);
+  UpdateLastClipboardAction(CItemData::FieldType::TOTPCONFIG);
+}
+
+/*!
+ * wxEVT_COMMAND_MENU_SELECTED event handler for ID_SHOWAUTHCODE
+ */
+
+void PasswordSafeFrame::OnShowAuthCodeClick(wxCommandEvent& WXUNUSED(evt))
+{
+  GetTotpBarPane().IsShown() ? HideTotpBar() : ShowTotpBar();
+}
+
 /*!
  * wxEVT_COMMAND_MENU_SELECTED event handler for ID_COPYNOTESFLD
  */
@@ -924,7 +958,7 @@ void PasswordSafeFrame::DoBrowse(CItemData &item, bool bAutotype)
 
 bool PasswordSafeFrame::LaunchBrowser(const wxString &csURL, const StringX & WXUNUSED(sxAutotype),
                              const std::vector<size_t> & WXUNUSED(vactionverboffsets),
-                             bool WXUNUSED(bDoAutotype))
+                             bool WXUNUSED(bDoAutotype)) const
 {
   /*
    * This is a straight port of DBoxMain::LaunchBrowser.  See the comments in that function
@@ -998,7 +1032,7 @@ bool PasswordSafeFrame::LaunchBrowser(const wxString &csURL, const StringX & WXU
 #endif
 
   if (!rc) {
-    wxMessageBox(errMsg, wxTheApp->GetAppName(), wxOK|wxICON_STOP, this);
+    wxMessageBox(errMsg, wxTheApp->GetAppName(), wxOK|wxICON_STOP, const_cast<PasswordSafeFrame*>(this));
   }
   return rc;
 }
@@ -1142,11 +1176,20 @@ void PasswordSafeFrame::DoPasswordQRCode(CItemData* item)
 {
 #ifndef NO_QR
   if (item) {
+    auto gtu = item->GetTitle();
+    if (!item->GetGroup().empty()) {
+      gtu = item->GetGroup() + _T(':') + gtu;
+    }
+    if (!item->GetUser().empty()) {
+      gtu += _T(':') + item->GetUser();
+    }
+
+    if (!item->GetGroup().empty() || !item->GetUser().empty()) {
+      gtu = _T('[') + gtu + _T(']');
+    }
+
     ShowModalAndGetResult<QRCodeDlg>(this, item->GetPassword(),
-            towxstring(CItemData::FieldName(CItem::PASSWORD)) + _T(" of ") +
-            towxstring(item->GetGroup()) +
-            _T('[') + towxstring(item->GetTitle()) + _T(']') +
-            _T(':') + towxstring(item->GetUser()));
+            _("Password of ") + towxstring(gtu));
   }
 #endif
 }
