@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2024 Rony Shapiro <ronys@pwsafe.org>.
+ * Copyright (c) 2003-2025 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -91,7 +91,7 @@ int PasswordSafeFrame::New()
     wxString msg(_("Do you want to save changes to the password database:"));
     msg += wxT("\n");
     msg += m_core.GetCurFile().c_str();
-    wxMessageDialog mbox(this, msg, "Save changes?", wxCANCEL | wxYES_NO | wxICON_QUESTION);
+    wxMessageDialog mbox(this, msg, _("Save changes?"), wxCANCEL | wxYES_NO | wxICON_QUESTION);
     mbox.SetYesNoLabels(_("Save"), _("Discard"));
     rc = mbox.ShowModal();
     switch (rc) {
@@ -138,6 +138,7 @@ int PasswordSafeFrame::New()
   m_sysTray->SetTrayStatus(SystemTray::TrayStatus::UNLOCKED);
   m_RUEList.ClearEntries();
   wxGetApp().recentDatabases().AddFileToHistory(towxstring(cs_newfile));
+  CreateMenubar(); // Recreate the menu with updated list of most recently used DBs
   ResetFilters();
   GetSearchBarPane().Hide(); // There is nothing to search for in an empty database
   m_AuiManager.Update();
@@ -242,13 +243,18 @@ void PasswordSafeFrame::OnOpenClick(wxCommandEvent& WXUNUSED(evt))
   int rc = DoOpen(_("Open Password Database"));
 
   if (rc == PWScore::SUCCESS) {
-    m_core.ResumeOnDBNotification();
-    CreateMenubar(); // Recreate the menu with updated list of most recently used DBs
-    UpdateSearchBarVisibility();
-    m_AuiManager.Update();
+    FinishGoodOpen();
   }
 }
 
+void PasswordSafeFrame::FinishGoodOpen()
+{
+  m_core.ResumeOnDBNotification();
+  CreateMenubar(); // Recreate the menu with updated list of most recently used DBs
+  UpdateSearchBarVisibility();
+  m_AuiManager.Update();
+}
+
 /*!
  * wxEVT_COMMAND_MENU_SELECTED event handler for wxID_CLOSE
  */
@@ -272,11 +278,19 @@ void PasswordSafeFrame::OnUnlockSafe(wxCommandEvent&)
  * wxEVT_COMMAND_MENU_SELECTED event handler for ID_MENU_CLEAR_MRU
  */
 
-void PasswordSafeFrame::OnClearRecentHistory(wxCommandEvent& evt)
+void PasswordSafeFrame::OnClearRecentHistory(wxCommandEvent& WXUNUSED(evt))
 {
-  UNREFERENCED_PARAMETER(evt);
-  wxGetApp().recentDatabases().Clear();
-  CreateMenubar(); // Recreate the menu with cleared list of most recently used DBs
+  wxMessageDialog dialog(
+    this,
+    _("Do you want to clear the list of recently opened databases?"),
+    _("Clear Recent Database List"),
+    wxICON_EXCLAMATION|wxOK|wxCANCEL|wxCANCEL_DEFAULT
+  );
+  dialog.SetOKLabel(_("Clear"));
+  if (dialog.ShowModal() == wxOK) {
+    wxGetApp().recentDatabases().Clear();
+    CreateMenubar(); // Recreate the menu with cleared list of most recently used DBs
+  }
 }
 
 ///////////////////////////////////////////////////////////////////////////////////////////////////
@@ -318,7 +332,7 @@ int PasswordSafeFrame::Save(SaveType savetype /* = SaveType::INVALID*/)
     m_tree->SaveGroupDisplayState();
   }
 
-  if (!m_core.IsDbOpen())
+  if (!m_core.IsDbFileSet())
     return SaveAs();
 
   switch (m_core.GetReadFileVersion()) {
@@ -396,7 +410,7 @@ int PasswordSafeFrame::Save(SaveType savetype /* = SaveType::INVALID*/)
 
   if (rc != PWScore::SUCCESS) { // Save failed!
     // Restore backup, if we have one
-    if (!bu_fname.empty() && m_core.IsDbOpen())
+    if (!bu_fname.empty() && m_core.IsDbFileSet())
       pws_os::RenameFile(bu_fname, m_core.GetCurFile().c_str());
     // Show user that we have a problem
     DisplayFileWriteError(rc, m_core.GetCurFile());
@@ -439,18 +453,18 @@ int PasswordSafeFrame::SaveAs()
   if(cf.empty()) {
     cf = wxT("pwsafe"); // reasonable default for first time user
   }
-  wxString v3FileName = towxstring(PWSUtil::GetNewFileName(cf.c_str(), DEFAULT_SUFFIX));
+  wxString NewFileName = towxstring(PWSUtil::GetNewFileName(cf.c_str(), curver == PWSfile::V40 ? V4_SUFFIX : V3_SUFFIX));
 
-  wxString title = (!m_core.IsDbOpen()? _("Choose a name for the current (Untitled) database:") :
+  wxString title = (!m_core.IsDbFileSet()? _("Choose a name for the current (Untitled) database:") :
                                     _("Choose a new name for the current database:"));
-  wxFileName filename(v3FileName);
+  wxFileName filename(NewFileName);
   wxString dir = filename.GetPath();
   if (dir.empty())
     dir = towxstring(PWSdirs::GetSafeDir());
 
   //filename cannot have the path
   wxFileDialog fd(this, title, dir, filename.GetFullName(),
-                  _("Password Safe Databases (*.psafe3; *.dat)|*.psafe3;*.dat|All files (*.*; *)|*.*;*"),
+                  curver == PWSfile::V40 ? _("Password Safe Databases (*.psafe4)|*.psafe4|All files (*.*; *)|*.*;*") : _("Password Safe Databases (*.psafe3; *.dat)|*.psafe3;*.dat|All files (*.*; *)|*.*;*"),
                    wxFD_SAVE | wxFD_OVERWRITE_PROMPT);
 
   if (fd.ShowModal() != wxID_OK) {
@@ -514,6 +528,7 @@ int PasswordSafeFrame::SaveAs()
   RefreshViews();
 
   wxGetApp().recentDatabases().AddFileToHistory(towxstring(newfile));
+  CreateMenubar();
 
   if (m_core.IsReadOnly()) {
     // reset read-only status (new file can't be read-only!)
@@ -566,11 +581,11 @@ int PasswordSafeFrame::SaveIfChanged()
   if ((m_bTSUpdated || m_core.HasDBChanged()) &&
       m_core.GetNumEntries() > 0) {
     wxString prompt(_("Do you want to save changes to the password database:"));
-    if (m_core.IsDbOpen()) {
+    if (m_core.IsDbFileSet()) {
       prompt += wxT("\n");
       prompt += m_core.GetCurFile().c_str();
     }
-    wxMessageDialog dlg(this, prompt, "Save changes?",
+    wxMessageDialog dlg(this, prompt, _("Save changes?"),
                         (wxICON_QUESTION | wxCANCEL | wxYES_NO));
     dlg.SetYesNoLabels(_("Save"), _("Discard"));
     int rc = dlg.ShowModal();
@@ -600,8 +615,12 @@ int PasswordSafeFrame::SaveIfChanged()
 
 struct ExportFullText
 {
-  static wxString EngGetTitle() {return CReport::ReportNames.find(IDSC_RPTEXPORTTEXT)->second;}
-  static wxString GetTitle() {return _("Export Text");}
+  static stringT GetTitle() {
+    stringT title;
+    LoadAString(title, IDSC_RPTEXPORTTEXT);
+    return title;
+  }
+
   static void MakeOrderedItemList(PasswordSafeFrame* frame, OrderedItemList& olist) {
     frame->FlattenTree(olist);
   }
@@ -640,11 +659,19 @@ struct ExportFullText
   static wxString GetTaskWord() {
     return _("export");
   }
+
+  static int GetReportNameId() {
+    return IDSC_RPTEXPORTTEXT;
+  }
 };
 
 struct ExportFullXml {
-  static wxString EngGetTitle() {return CReport::ReportNames.find(IDSC_RPTEXPORTXML)->second;}
-  static wxString GetTitle() {return _("Export XML");}
+  static stringT GetTitle() {
+    stringT title;
+    LoadAString(title, IDSC_RPTEXPORTXML);
+    return title;
+  }
+
   static void MakeOrderedItemList(PasswordSafeFrame* frame, OrderedItemList& olist) {
     frame->FlattenTree(olist);
   }
@@ -683,6 +710,10 @@ struct ExportFullXml {
   static wxString GetTaskWord() {
     return _("export");
   }
+
+    static int GetReportNameId() {
+    return IDSC_RPTEXPORTXML;
+  }
 };
 
 void PasswordSafeFrame::OnExportVx(wxCommandEvent& evt)
@@ -813,8 +844,13 @@ void PasswordSafeFrame::DoExportText()
           newfile = fd.GetPath().c_str();
           CReport rpt;
 
-          rpt.StartReport(IDSC_RPTEXPORTTEXT, sx_temp.c_str());
-          rpt.WriteLine(tostdstring(wxString(_("Exporting database: ")) << towxstring(sx_temp) << _(" to ") << newfile<< wxT("\r\n")));
+          rpt.StartReport(ExportType::GetReportNameId(), sx_temp.c_str());
+
+          auto db_info = wxString::Format(
+            _("Exporting database: %ls to %ls\r\n"),
+            towxstring(sx_temp), towxstring(newfile)
+          );
+          rpt.WriteLine(tostdstring(db_info));
 
           int rc = ExportType::Write(m_core, newfile, bsExport, subgroup_name, subgroup_object,
                                       subgroup_function, delimiter, numExported, &orderedItemList, &rpt);
@@ -1184,7 +1220,7 @@ void PasswordSafeFrame::DoImportKeePass(wxString filename)
     {
       wxMessageBox( wxString::Format(_("%ls\n\nCould not open file for reading!"), KPsFileName.GetData()),
                     _("File open error"), wxOK | wxICON_ERROR, this);
-      delete [] pcmd;
+      delete pcmd;
       break;
     }
     case PWScore::INVALID_FORMAT:
@@ -1202,7 +1238,7 @@ void PasswordSafeFrame::DoImportKeePass(wxString filename)
       msg << wxT("\n\n") << _("Do you wish to see a detailed report?");
       if (wxMessageBox(msg, _("Import failed"), wxYES_NO | wxICON_ERROR, this) == wxYES)
         ViewReport(rpt);
-      delete [] pcmd;
+      delete pcmd;
       break;
     }
     case PWScore::SUCCESS:
@@ -1312,7 +1348,7 @@ void PasswordSafeFrame::OnSynchronize(wxCommandEvent& evt)
 void PasswordSafeFrame::DoSynchronize(wxString filename)
 {
   // disable in read-only mode or empty
-  wxCHECK_RET(!m_core.IsReadOnly() && m_core.IsDbOpen() && m_core.GetNumEntries() != 0,
+  wxCHECK_RET(!m_core.IsReadOnly() && m_core.IsDbFileSet() && m_core.GetNumEntries() != 0,
                 wxT("Synchronize menu enabled for empty or read-only database!"));
 
   SyncWizard wiz(this, &m_core, filename);
