@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2024 Rony Shapiro <ronys@pwsafe.org>.
+ * Copyright (c) 2003-2025 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -225,8 +225,13 @@ void PasswordSafeFrame::RunShowReport(int iAction)
     ShowModalAndGetResult<ViewReportDlg>(this, &rpt, true);
   }
   else {
-    wxString tcAction = CReport::ReportNames.find(iAction)->second;
-    wxMessageBox(_(tcAction) + _(L" file \'") + rpt.GetFileName() + _(L"' not readable"), _("View Report"), wxOK|wxICON_ERROR);
+    stringT title;
+    stringT reportName;
+    LoadAString(reportName, iAction);
+    Format(title, _("View Report: %ls"), reportName.c_str());
+    stringT message;
+    Format(message, _(L"File \'%ls\' not readable"), rpt.GetFileName().c_str());
+    wxMessageBox(message, title, wxOK|wxICON_ERROR);
   }
 }
 
@@ -387,6 +392,7 @@ void PasswordSafeFrame::ApplyFilters()
 {
   m_FilterManager.CreateGroups();
   // Update and setting of filter state is not needed here, as update is done at the end of RefreshView()
+  m_tree->SetFilterActive(m_bFilterActive);
   RefreshViews();
 }
 
