@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2024 Rony Shapiro <ronys@pwsafe.org>.
+ * Copyright (c) 2003-2025 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -31,6 +31,12 @@
 #include "graphics/toolbar/new/copypassword_disabled.xpm"
 ////@end XPM images
 
+/*!
+ * PasswordPolicyDlg type definition
+ */
+
+IMPLEMENT_CLASS( PasswordPolicyDlg, wxDialog )
+
 /*!
  * PasswordPolicyDlg event table definition
  */
@@ -310,7 +316,7 @@ void PasswordPolicyDlg::CreateControls()
 
   m_pwpHexCtrl = new wxCheckBox( itemDialog1, ID_CHECKBOX9, _("Use hexadecimal digits only"), wxDefaultPosition, wxDefaultSize, 0 );
   m_pwpHexCtrl->SetValue(false);
-  m_pwpHexCtrl->Bind(wxEVT_MOTION, [&](wxMouseEvent& WXUNUSED(event)) { m_pwpHexCtrl->SetToolTip(_("0-9, a-f")); });
+  m_pwpHexCtrl->Bind(wxEVT_MOTION, [&](wxMouseEvent& WXUNUSED(event)) { m_pwpHexCtrl->SetToolTip(_T("0-9, a-f")); });
   m_pwMinsGSzr->Add(m_pwpHexCtrl, 0, wxALIGN_LEFT|wxALL, 5);
 
   /////////////////////////////////////////////////////////////////////////////
