@@ -227,6 +227,7 @@ void PasswordSafeSearch::OnDoSearchT(Iter begin, Iter end, Accessor afn)
 
   UpdateView();
   txtCtrl->SelectNone();
+  txtCtrl->SetInsertionPointEnd();
 
   // Replace the "Find" menu item under Edit menu by "Find Next" and "Find Previous"
   wxMenu* editMenu = nullptr;
@@ -674,6 +675,7 @@ void PasswordSafeSearch::Activate()
   }
 
   UpdateStatusAreaWidth();
+  wxDynamicCast(FindControl(ID_FIND_EDITBOX), wxSearchCtrl)->SelectAll();
 }
 
 void PasswordSafeSearch::SetFocusIntoEditField()
@@ -706,7 +708,7 @@ SearchPointer& SearchPointer::operator++()
     m_currentIndex++;
     if (m_currentIndex == m_indices.end()) {
       m_currentIndex = m_indices.begin();
-      PrintLabel(_("Search hit bottom, continuing at top").c_str());
+      PrintLabel(_("Search hit bottom, continuing at top.").c_str());
     }
     else {
       PrintLabel();
@@ -724,7 +726,7 @@ SearchPointer& SearchPointer::operator--()
   if (!m_indices.empty()) {
     if (m_currentIndex == m_indices.begin()) {
       m_currentIndex = --m_indices.end();
-      PrintLabel(_("Search hit top, continuing at bottom").c_str());
+      PrintLabel(_("Search hit top, continuing at bottom.").c_str());
     }
     else {
       m_currentIndex--;
@@ -749,10 +751,9 @@ void SearchPointer::PrintLabel(const TCHAR* prefix /*= 0*/)
   else {
     // need a const object so we get both args to distance() as const iterators
     const SearchIndices& idx = m_indices;
-    m_label.Clear();
-    m_label << std::distance(idx.begin(), m_currentIndex)+1 << '/' << m_indices.size() << wxT(" matches");
+    m_label.Printf(_("%td/%zu matches"), std::distance(idx.begin(), m_currentIndex)+1, m_indices.size());
     if (prefix) {
-      m_label = wxString(prefix) + wxT(".  ") + m_label;
+      m_label = wxString(prefix) + "  " + m_label;
     }
   }
 }
