@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2024 Rony Shapiro <ronys@pwsafe.org>.
+ * Copyright (c) 2003-2025 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -354,6 +354,10 @@ void TreeCtrl::UpdateGUI(UpdateGUICommand::GUI_Action ga, const pws_os::CUUID &e
       ASSERT(item != nullptr);
       if(!m_bFilterActive) {
         AddItem(*item);
+        auto itemId = Find(*item);
+        if (itemId.IsOk()) {
+          EnsureVisible(itemId);
+        }
       }
       break;
     case UpdateGUICommand::GUI_DELETE_ENTRY:
@@ -566,11 +570,16 @@ wxString TreeCtrlBase::ItemDisplayString(const CItemData &item) const
   }
 
   if (item.IsProtected()) { 
-    disp += wxT(" #");
-#ifdef NOTYET
-    wxUniChar padlock(0x1f512);
-    disp +=padlock;
-#endif
+    // disp += wxT(" #"); - this is what we displayed before unicode...
+
+    const wxUniChar padlock(0x1f512);
+    disp += padlock;
+  }
+
+  if (item.HasAttRef() || item.HasAttachment())
+  {
+    const wxUniChar paperclip(0x1f4ce);
+    disp += paperclip;
   }
 
   return disp;
@@ -944,12 +953,38 @@ void TreeCtrlBase::SelectItem(const CUUID & uuid)
   wxTreeItemId id = Find(uuid_array);
   if (id.IsOk()) {
     wxTreeItemId parent = GetItemParent(id);
-    if(parent.IsOk() && (parent != GetRootItem()) && ! IsExpanded(parent))
+    if(parent.IsOk() && (parent != GetRootItem()) && !IsExpanded(parent)) {
       Expand(parent);
+    }
     ::wxSafeYield();
-    EnsureVisible(id);
+    // Determine an item's height based on its surrounding rectangle.
+    // To obtain a surrounding rectangle, the element must be visible.
+    auto firstVisibleItem = GetFirstVisibleItem();
+    if (firstVisibleItem.IsOk()) {
+      wxRect boundingRect;
+      if (GetBoundingRect(firstVisibleItem, boundingRect)) {
+        auto itemHeight = boundingRect.GetHeight();
+        auto viewHeight = GetClientSize().GetHeight();
+
+        // Scroll the tree view up one more position (to the next sibling element)
+        // if there is sufficient space (at least two items) in the tree view.
+        if (2 /*visible items*/ * itemHeight < viewHeight) {
+          auto sibling = searchSibling(id);
+          EnsureVisible(sibling.IsOk() ? sibling : id);
+        }
+        else {
+          // If there is not enough space in the tree view, then scroll only
+          // to the search result instead of to the next sibling element.
+          EnsureVisible(id);
+        }
+      }
+    }
+    else {
+      // By default, scroll directly to the search results.
+      EnsureVisible(id);
+    }
     ::wxSafeYield();
-    
+
     wxTreeCtrl::SelectItem(id);
   }
 }
@@ -1432,7 +1467,7 @@ void TreeCtrl::OnEndDrag(wxTreeEvent& evt)
 
     wxTreeItemId itemDst = evt.GetItem();
     auto parentOfDragItem = GetItemParent(m_drag_item);
-    auto sxSrcGroupName = tostringx(GetItemText(parentOfDragItem));
+    auto sxSrcGroupName = tostringx(GetItemGroup(m_drag_item));
     StringX sxDstGroupName;
     bool dragItemLeavesEmptyGroup = (ItemIsGroup(parentOfDragItem) && GetChildrenCount(parentOfDragItem) == 1);
     bool isDestinationEmptyGroup = false;
@@ -1455,7 +1490,7 @@ void TreeCtrl::OnEndDrag(wxTreeEvent& evt)
           itemDst = GetItemParent(itemDst);
         }
         else {                                    // On group, the group needs to be removed, if it is an empty group
-          sxDstGroupName = tostringx(GetItemText(itemDst));
+          sxDstGroupName = tostringx(GetItemGroup(itemDst));
           isDestinationEmptyGroup = m_core.IsEmptyGroup(sxDstGroupName);
         }
         if(IsDescendant(itemDst, m_drag_item)) {  // Do not drag and drop into the moved tree
@@ -1471,8 +1506,10 @@ void TreeCtrl::OnEndDrag(wxTreeEvent& evt)
       // If the drag'd item leaves an empty group in the tree
       // the group needs to be created as such in the database.
       if (dragItemLeavesEmptyGroup) {
+        auto emptyGroups = m_core.GetEmptyGroups();
+        emptyGroups.push_back(sxSrcGroupName);
         commands->Add(
-          DBEmptyGroupsCommand::Create(&m_core, sxSrcGroupName, DBEmptyGroupsCommand::EG_ADD)
+          DBEmptyGroupsCommand::Create(&m_core, emptyGroups, DBEmptyGroupsCommand::EG_REPLACEALL)
         );
       }
       // If the destination was an empty group in the tree it is not empty now
@@ -2608,7 +2645,7 @@ void TreeCtrl::AddDnDEntries(MultiCommands *pmCmd, DnDObList &dnd_oblist, String
         pl.base_uuid = CUUID::NullUUID();
       }
     } else {
-        m_core.ParseBaseEntryPWD(cs_tmp, pl);
+        m_core.ParseAliasPassword(cs_tmp, pl);
     }
     
     if (pl.ibasedata > 0) {
@@ -2622,14 +2659,14 @@ void TreeCtrl::AddDnDEntries(MultiCommands *pmCmd, DnDObList &dnd_oblist, String
         if(pl.TargetType == CItemData::ET_ALIAS) {
           // This base is in fact an alias. ParseBaseEntryPWD already found 'proper base'
           // So dropped entry will point to the 'proper base' and tell the user.
-          wxString cs_msg = wxString::Format(_("Dropped entry '%s:%s:%s' password points to an entry that is also an alias.\n\nThe dropped entry now references that entry's base entry."),
+          wxString cs_msg = wxString::Format(_("Dropped entry '%ls:%ls:%ls' password points to an entry that is also an alias.\n\nThe dropped entry now references that entry's base entry."),
                             sxgroup.c_str(),
                             sxtitle.c_str(),
                             sxuser.c_str());
           wxMessageBox(cs_msg, _("Drag and Drop failed"), wxOK);
         } else if(pl.TargetType != CItemData::ET_NORMAL && pl.TargetType != CItemData::ET_ALIASBASE) {
           // Only normal or alias base allowed as target
-          wxString cs_msg = wxString::Format(_("This alias's target [%s:%s:%s] is not a normal entry."),
+          wxString cs_msg = wxString::Format(_("This alias's target [%ls:%ls:%ls] is not a normal entry."),
                             sxgroup.c_str(),
                             sxtitle.c_str(),
                             sxuser.c_str());
@@ -2646,7 +2683,7 @@ void TreeCtrl::AddDnDEntries(MultiCommands *pmCmd, DnDObList &dnd_oblist, String
       } else if(pl.InputType == CItemData::ET_SHORTCUT) {
         if(pl.TargetType != CItemData::ET_NORMAL && pl.TargetType != CItemData::ET_SHORTCUTBASE) {
           // Only normal or shortcut base allowed as target
-          wxString cs_msg = wxString::Format(_("This shortcut's target [%s:%s:%s] is not a normal entry."),
+          wxString cs_msg = wxString::Format(_("This shortcut's target [%ls:%ls:%ls] is not a normal entry."),
                             sxgroup.c_str(),
                             sxtitle.c_str(),
                             sxuser.c_str());
@@ -2811,3 +2848,36 @@ void TreeCtrlBase::setNodeAsEmptyIfNeeded(const wxTreeItemId item)
     wxTreeCtrl::SetItemImage(item, EMPTY_NODE_II); // Empty Group shall show the empty node icon
   }
 }
+
+/**
+ * Searches for a subsequent or sibling element (password element / group element)
+ * in the tree view at the same level. If no element is found, the search continues
+ * one level higher until the root element is reached.
+ * 
+ * The goal is to find the next visual element below the target element so that it
+ * appears visually positioned slightly higher. See also the documentation for
+ * wxUtilities::IsAdvancedScrollEnabled().
+ *
+ * @param itemId refers to the element from which the search should start.
+ * @return the found sibling element or an invalid tree element if the search failed.
+ */
+wxTreeItemId TreeCtrlBase::searchSibling(const wxTreeItemId& itemId) const
+{
+  // If a sibling element of the specified element was found,
+  // then the search was successful and can be stopped.
+  auto nextSibling = wxTreeCtrl::GetNextSibling(itemId);
+  if (nextSibling.IsOk()) {
+    return nextSibling;
+  }
+
+  // If the root element has been reached, then the search
+  // was not successful and can be stopped. The returned
+  // tree item id is invalid, which indicates a failed search.
+  auto parentId = GetItemParent(itemId);
+  if (parentId == GetRootItem()) {
+    return wxTreeItemId();
+  }
+
+  // The search continues further up the hierarchy.
+  return searchSibling(parentId);
+}
