@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2024 Rony Shapiro <ronys@pwsafe.org>.
+ * Copyright (c) 2003-2025 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -27,7 +27,8 @@ public:
 protected:
   ViewReportDlg(wxWindow *parent, CReport* pRpt, bool fromFile);
   void OnSave(wxCommandEvent& event);
-  void OnRemove(wxCommandEvent& event);
+  void OnSaveAs(wxCommandEvent& event);
+  void OnDelete(wxCommandEvent& event);
   void OnClose(wxCommandEvent& event);
   void OnCopy(wxCommandEvent& event);
 };
